#!/bin/bash

# Exit on error
set -e
set -u

# Check args
if [ "$#" -ne 3 ]; then
  echo "Usage: $0 <subscribed-participants-csv> <participant-allocation> <validator-allocation>"
  echo "Example: $0 /home/user/subscribed-participants.csv 1000000 1000000000000"
  exit 1
fi

SUBSCRIBED_PARTICIPANTS_CSV="$1"
PARTICIPANT_ALLOCATION="$2"
VALIDATOR_ALLOCATION="$3"
STAGE1_GENESIS_DIR=stage1-genesis

# Create a temporary target directory
mkdir -p $STAGE1_GENESIS_DIR

# --------

# Parse the input CSV file to create stage1 onboarding module state
# Input CSV columns:
# 5: laconic_address, 6: nitro_address, 7: role, 8: hashed_subscriber_id, 11: funding_amount

stage1_participants_file="$STAGE1_GENESIS_DIR/stage1-participants.json"
jq -R -s --arg participant_allocation "$PARTICIPANT_ALLOCATION" --arg validator_allocation "$VALIDATOR_ALLOCATION" -f <(cat <<'EOF'
# Split the input into lines
split("\n") |
# Skip the first line (header) and filter out empty lines
.[1:] | map(select(length > 0)) |
# For each line, split it into fields by comma
map(split(",") | map(gsub("^\"|\"$"; ""))) |
# Create objects from the relevant fields
map({
  cosmos_address: .[5],
  nitro_address: .[6],
  role: .[7],
  kyc_id: .[8]
})
EOF
) < "$SUBSCRIBED_PARTICIPANTS_CSV" > "$stage1_participants_file"

echo "Exported participants for stage1 onboarding module state"

# --------

# Parse the input CSV file to calculate allocations on stage1
# Use provided input amounts if funding amount not present in the CSV

stage1_allocations_file="$STAGE1_GENESIS_DIR/stage1-allocations.json"
jq -R -s --arg participant_allocation "$PARTICIPANT_ALLOCATION" --arg validator_allocation "$VALIDATOR_ALLOCATION" -f <(cat <<'EOF'
# Split the input into lines
split("\n") |
# Skip the first line (header) and filter out empty lines
.[1:] | map(select(length > 0)) |
# For each line, split it into fields by comma
map(split(",") | map(gsub("^\"|\"$"; ""))) |
# Create objects from the relevant fields
map({
  cosmos_address: .[5],
  balance: (
    if .[11] == "" or .[11] == null then
      if .[7] == "validator" then $validator_allocation else $participant_allocation end
    else .[11] end
  )
})
EOF
) < "$SUBSCRIBED_PARTICIPANTS_CSV" > "$stage1_allocations_file"

echo "Calculated allocations for stage 1 chain"

# --------

# Run a script with cerc/laconicd:local to generate the genesis file
# with onboarding module state and given allocations
docker run -it \
  -v ./$STAGE1_GENESIS_DIR:/root/.laconicd \
  -v ./scripts:/scripts \
  cerc/laconicd:local bash -c "/scripts/genesis.sh"

# Copy over the genesis file to output folder
OUTPUT_DIR=output
mkdir -p $OUTPUT_DIR
cp ./$STAGE1_GENESIS_DIR/config/genesis.json $OUTPUT_DIR/genesis.json

echo "Genesis file for stage1 written to $OUTPUT_DIR/genesis.json"

# --------

# Clean up
echo "Please remove the temporary data directory: sudo rm -rf $STAGE1_GENESIS_DIR"
