#!/bin/bash

# Exit on error
set -e
set -u

# Check args
if [ "$#" -ne 3 ]; then
  echo "Usage: $0 <stage0-deployment-dir-absolute> <participant-allocation> <validator-allocation>"
  echo "Example: $0 /home/user/stage0-deployment 1000000 1000000000000"
  exit 1
fi

STAGE0_DEPLOYMENT_DIR="$1"
PARTICIPANT_ALLOCATION="$2"
VALIDATOR_ALLOCATION="$3"
STAGE1_GENESIS_DIR=stage1-genesis

# Create a temporary target directory
mkdir -p $STAGE1_GENESIS_DIR

# --------

# Export participants from onboarding module state on stage0 laconicd chain
# Use cerc/laconicd-stage0:local image for stage0 laconicd build

stage1_participants_file="$STAGE1_GENESIS_DIR/stage1-participants.json"
docker run -it \
  -v ${STAGE0_DEPLOYMENT_DIR}/data/laconicd-data:/root/stage0-deployment/.laconicd \
  cerc/laconicd-stage0:local bash -c "laconicd export --home /root/stage0-deployment/.laconicd" \
  | jq .app_state.onboarding.participants > "$stage1_participants_file"

stage1_participants=$(cat "$stage1_participants_file")

echo "Exported participants for stage1 onboarding module state"

# --------

# For each participant, using ETH account holdings, figure out balance allocation for stage1
stage1_allocations_file="$STAGE1_GENESIS_DIR/stage1-allocations.json"
jq -n --argjson participants "$stage1_participants" --arg participant_allocation "$PARTICIPANT_ALLOCATION" --arg validator_allocation "$VALIDATOR_ALLOCATION" '
[
  $participants[] |
  {
    cosmos_address: .cosmos_address,
    balance: (if .role == "validator" then $validator_allocation else $participant_allocation end)
  }
]' > $stage1_allocations_file

echo "Calculated allocations for stage 1 chain"

# --------

# Run a script with cerc/laconicd:local to generate the genesis file
# with onboarding module state and given allocations
docker run -it \
  -v ./$STAGE1_GENESIS_DIR:/root/.laconicd \
  -v ./scripts:/scripts \
  cerc/laconicd:local bash -c "/scripts/genesis.sh"

# Copy over the genesis file to output folder
OUTPUT_DIR=output
mkdir -p $OUTPUT_DIR
cp ./$STAGE1_GENESIS_DIR/config/genesis.json $OUTPUT_DIR/genesis.json

echo "Genesis file for stage1 written to $OUTPUT_DIR/genesis.json"

# --------

# Clean up
echo "Please remove the temporary data directory: sudo rm -rf $STAGE1_GENESIS_DIR"
